//
//  CalculatorBrain.m
//  Stanford_Cal_Assignment_1
//
//  Created by LauRi3A on 1/07/12.
//  Copyright (c) 2012 Beginning. All rights reserved.
//

#import "CalculatorBrain.h"

@interface CalculatorBrain ()
@property (nonatomic, strong) NSMutableArray *programStack;
@property (nonatomic) NSString *description;
@end 

@implementation CalculatorBrain

@synthesize programStack = _programStack;
@synthesize description = _description;

-(NSMutableArray *)programStack
{
    if (_programStack == nil) _programStack = [[NSMutableArray alloc] init];
    return _programStack;
}

-(void)pushOperand:(double)operand
{
    [self.programStack addObject:[NSNumber numberWithDouble:operand]];
}

-(id)program
{
    return [self.programStack copy];
}

+(NSString *)descriptionOfProgram:(id)program
{
    
    
    return @"Assignment 2";
}

-(double)performOperation:(NSString *)operation
{
    if ([operation isEqualToString:@"C"]) {
        self.programStack = nil;
        self.description = @"";
        return 0;
    }
    else
    {
        [self.programStack addObject:operation];
        return [[self class] runProgram:self.program];
    }
}


+(double)popOperandOffStack:(NSMutableArray *)stack
{
    double result = 0;
    
    id topOfStack = [stack lastObject];
    if (topOfStack) [stack removeLastObject];
    
    if ([topOfStack isKindOfClass:[NSString class]]) {
        NSString *operation = topOfStack;
        if ([operation isEqualToString:@"sin"]) {
            result = sin([self popOperandOffStack:stack]);
        }
        else if ([operation isEqualToString:@"cos"]) {
            result = cos([self popOperandOffStack:stack]);
        }
        else if ([operation isEqualToString:@"sqrt"]) {
            result = sqrt([self popOperandOffStack:stack]);
        }
        else if ([operation isEqualToString:@"π"]) {
            result = M_PI;
        }
        else if ([operation isEqualToString:@"±"]) {
            result = [self popOperandOffStack:stack] * -1;
        }
        else
        {
            if ([operation isEqualToString:@"+"]) {
                result = [self popOperandOffStack:stack] + [self popOperandOffStack:stack];
            } else if ([operation isEqualToString:@"*"]) {
                result = [self popOperandOffStack:stack] * [self popOperandOffStack:stack];
            } else if ([operation isEqualToString:@"-"]) {
                result = - [self popOperandOffStack:stack] + [self popOperandOffStack:stack];
            } else if ([operation isEqualToString:@"/"]) {
                double devisor = [self popOperandOffStack:stack];
                if (devisor) result = [self popOperandOffStack:stack] / devisor;
            }
        }
    }
    else if ([topOfStack isKindOfClass:[NSNumber class]]) {
        result = [topOfStack doubleValue];
    }
    
    return result;
}

+ (double) runProgram:(id)program
  usingVariableValues:(NSDictionary *)variableValues {
    
    // Ensure program is an NSArray
    if (![program isKindOfClass:[NSArray class]]) return 0;
    
    NSMutableArray *stack= [program mutableCopy];
    
    // For each item in the program
    for (int i=0; i < [stack count]; i++) {
        id obj = [stack objectAtIndex:i];
        
        // See whether we think the item is a variable
        if ([obj isKindOfClass:[NSString class]] && ![self isOperation:obj]) {
            id value = [variableValues objectForKey:obj];
            // If value is not an instance of NSNumber, set it to zero
            if (![value isKindOfClass:[NSNumber class]]) {
                value = [NSNumber numberWithInt:0];
            }
            // Replace program variable with value.
            [stack replaceObjectAtIndex:i withObject:value];
        }
    }
    // Starting popping off the stack
    return [self popOperandOffStack:stack];
}

+ (NSSet *)variablesUsedInProgram:(id)program {
    
    // Ensure program is an NSArray
    if (![program isKindOfClass:[NSArray class]]) return nil;
    
    NSMutableSet *variables = [NSMutableSet set];
    
    // For each item in the program
    for (id obj in program) {
        // If we think it's a variable add it to the variables set
        if ([obj isKindOfClass:[NSString class]] && ![self isOperation:obj]) {
            [variables addObject:obj];
        }
    }
    // Return nil if we don't have any variables
    if ([variables count] == 0) return nil;
    else return [variables copy];
}

-(void)pushVariable:(NSString *)variable {
    [self.programStack addObject:variable];
}

-(void)pushOperation:(NSString *)operation {
    [self.programStack addObject:operation];
}

+(BOOL)isOperation:(NSString *)operation {
    NSSet *operationSet = [NSSet setWithObjects:@"+", @"-", @"*", @"/", @"sin", @"cos", @"sqrt", @"π", @"±", nil];
    
    return [operationSet containsObject:operation];
}

+(double)runProgram:(id)program
{
    NSMutableArray *stack;
    if ([program isKindOfClass:[NSArray class]]) {
        stack = [program mutableCopy];
    }
    
    return [self runProgram:program usingVariableValues:nil];
}

@end
